function a = setupm(flag);
% setupm( [flag] )
% Demonstrates how to use the m-packge and tests if the m-package works correctly
% if flag is 1, then the functions in the package are tested.
%
% Written by: tommsch, 2018

%#ok<*NOPRT>
%#ok<*ASGLU>
%#ok<*NASGU>
if(isequal(flag,'selftest')); a=1; return; end; %needed for the selftest of parsem
if(nargin>=1 && flag); selftestm; return; end;

fprintf('Either start the function with argument <1> or read the source code of this file.\n'); 
return; 
2; %#ok<UNRCH>

end

function selftestm
    fprintf('The m-package will be tested. All output on the screen should be followed by an explanation.\n');
    fprintf('If there is some error or un-commented output, the test fails.\n\n');

    vprintf('allm\n');
    AA=[1 1 2;0 1 1; 1 1 1];
    BB=[0 1 2;0 1 1; 0 1 1];
    if(~isequal(allm(),1)); fprintf('ERROR: allm() is broken.\n\n'); end;
    if(~isequal(allm(AA),0)); fprintf('ERROR: allm() is broken.\n\n'); end;
    if(~isequal(allm(AA,1),[0 1 1])); fprintf('ERROR: allm() is broken.\n\n'); end;
    if(~isequal(allm(AA,2),[1;0;1])); fprintf('ERROR: allm() is broken.\n\n'); end;
    if(~isequal(allm(AA,[1 2]),0)); fprintf('ERROR: allm() is broken.\n\n'); end;
    
    vprintf('anym\n');
    if(~isequal(anym(),0)); fprintf('ERROR: anym() is broken.\n\n'); end;
    if(~isequal(anym(BB),1)); fprintf('ERROR: anym() is broken.\n\n'); end;
    if(~isequal(anym(BB,1),[0 1 1])); fprintf('ERROR: anym() is broken.\n\n'); end;
    if(~isequal(anym(BB,2),[1;1;1])); fprintf('ERROR: anym() is broken.\n\n'); end;
    if(~isequal(anym(BB,[1 2]),1)); fprintf('ERROR: anym() is broken.\n\n'); end;
    
    
    vprintf('sph2cartm/cart2sphm\n');
    if(norm([2 3 1]-sph2cartm(cart2sphm([2 3 1])))>10e-12); fprintf('ERROR: sph2cartm()/cart2sphm() is broken.\n\n'); end;
    
    vprintf('sph2cartm2/cart2sphm2\n');
    if(abs(norm(sph2cartm2(cart2sphm2([2 3 1])))-1)>10e-12); fprintf('ERROR: sph2cartm()/cart2sphm() is broken.\n\n'); end;
    
    vprintf('convm\n');
    seq1=convm([1 2]);
    if(norm(seq1-[1 2])>10e-12); fprintf('ERROR: convm() is broken.\n\n'); end;        
    seq1=convm([1 1],[1 2]);
    if(norm(seq1-[1 3 2])>10e-12); fprintf('ERROR: convm() is broken.\n\n'); end;    
    seq1=convm([1 1]',[1 2]');
    if(norm(seq1-[1 3 2]')>10e-12); fprintf('ERROR: convm() is broken.\n\n'); end;        
    seq1=convm(sym([1 1]),[1 2]);
    if(norm(seq1-[1 3 2])>10e-12); fprintf('ERROR: convm() is broken.\n\n'); end;    
    seq1=convm([1 2 1],[1; 3; 1],'mult',[2 1],'normalize');
    if(norm(seq1-1/80*[1 4 6 4 1;3 12 18 12 3;1 4 6 4 1])>10e-12); fprintf('ERROR: convm() is broken.\n\n'); end;
    seq1=convm([1 2 1],[1; 3; 1],'outer','normalize');
    if(norm(seq1-1/20*[1 3 1; 2 6 2; 1 3 1])>10e-12); fprintf('ERROR: convm() is broken.\n\n'); end;
    convm(randn(2,2,2),randn(2,2,2),randn(2,2,2),randn(2,2,2));
    
    vprintf('dec2basem\n');
    num=dec2basem([10; 1],[1 2; -1 2]);
    if(~isequal(num,[1 1 2;1 1 1])); fprintf('ERROR: dec2basem() is broken.\n\n'); end;
    num=dec2basem(-1,2,5);
    if(~isequal(num(1,1:5),[1 1 1 1 1])); fprintf('ERROR: dec2basem() is broken.\n\n'); end;
    num=dec2basem(3,2,5);
    if(~isequal(num,[0 0 0 1 1])); fprintf('ERROR: dec2basem() is broken.\n\n'); end;
    
    vprintf('factorialm\n');
    if(~isequal(factorialm([3 4]),144)); fprintf('ERROR: factorialm() is broken.\n\n'); end;
    
    vprintf('ind2subm\n');
    ind2subm([2 1 3],[1 6]);
    
    vprintf('isvectorm\n');
    isvectorval=[isvectorm(ones(1,1,3)) isvectorm(ones(0)) isvector(ones(1,1,3)) isvector(ones(0))];
    if(~isequal(isvectorval,[true false false false])); fprintf('ERROR: isvectorm() is broken.\n\n'); end;
    
    vprintf('kronm\n');
    val=kronm([1 2 3],[2; 3]);
    if(~isequal(val,[2 4 6; 3 6 9])); fprintf('ERROR: kronm() is broken.\n\n'); end;
    val=kronm([],[],[]);
    if(~isequal(val,[])); fprintf('ERROR: kronm() is broken.\n\n'); end;
    
    vprintf('lcmm\n');
    if(~isequal(lcmm(10,15,20),60)); fprintf('ERROR: lcmm() is broken.\n\n'); end;;
    if(~isequal(lcmm(1),1)); fprintf('ERROR: lcmm() is broken.\n\n'); end;;
    
    
    vprintf('maxm\n');
    if(~isequal(maxm(),-Inf)); fprintf('ERROR: minm() is broken.\n\n'); end;;
    if(~isequal(maxm([1 2 2; 2 3 4; 2 1 3;4 5 NaN]),5)); fprintf('ERROR: maxm() is broken.\n\n'); end;;
    if(~isequal(maxm([1 2 2; 2 3 4; 2 1 3],1),[2 3 4])); fprintf('ERROR: maxm() is broken.\n\n'); end;;
    
    vprintf('minm\n');
    if(~isequal(minm(),Inf)); fprintf('ERROR: minm() is broken.\n\n'); end;;
    if(~isequal(minm([1 2 2; 2 3 4; 2 1 3;4 5 NaN]),1)); fprintf('ERROR: minm() is broken.\n\n'); end;;
    if(~isequal(minm([1 2 2; 2 3 4],2),[1;2])); fprintf('ERROR: minm() is broken.\n\n'); end;;
    
    vprintf('nchoosekm\n');
    if(~isequal(nchoosekm([10 5],[5 2]),2520)); fprintf('ERROR: nchoosekm() is broken.\n\n'); end;;
    
    vprintf('ndimsm\n');
    if(~isequal(ndimsm([1; 2; 3]),1)); fprintf('ERROR: ndimsm() is broken.\n\n'); end;;
    
    vprintf('onesm\n');
    if(~isequal(onesm,[])); fprintf('ERROR: onesm() is broken.\n\n'); end;;
    if(~isequal(onesm([]),[])); fprintf('ERROR: onesm() is broken.\n\n'); end;;
    if(~isequal(onesm(1,2),[1 1])); fprintf('ERROR: onesm() is broken.\n\n'); end;;
    if(~isequal(onesm(3),[1;1;1])); fprintf('ERROR: onesm() is broken.\n\n'); end;;
    
    vprintf('padarraym\n');
    if(~isequal(padarraym([1],1),[0;1;0])); fprintf('ERROR: padarraym() is broken.\n\n'); end;
    if(~isequal(padarraym([1;2],1),[0;1;2;0])); fprintf('ERROR: padarraym() is broken.\n\n'); end;
    if(~isequal(padarraym([1 2],1,'pre'),[0 0 0;0 1 2])); fprintf('ERROR: padarraym() is broken.\n\n'); end;
    if(~isequal(padarraym([1 2],1,'post'),[1 2 0;0 0 0])); fprintf('ERROR: padarraym() is broken.\n\n'); end;
    if(~isequal(padarraym([1 2],1),[0 0 0 0;0 1 2 0;0 0 0 0])); fprintf('ERROR: padarraym() is broken.\n\n'); end; 
    if(~isequal(padarraym([1 2;3 4],0),[1 2;3 4])); fprintf('ERROR: padarraym() is broken.\n\n'); end;
    if(~isequal(padarraym({1 2;@sum 4},1),{[] [] [] [];[] 1 2 [];[] @sum 4 [];[] [] [] []})); fprintf('ERROR: padarraym() is broken.\n\n'); end;
    
    vprintf('parsem\n');
    [v,arg]=parsem('selftest',{'asd','selftest',2},@setupm);
    if(~isequal(v,2) || ~isequal(arg, {'asd'})); vprintf('ERROR: parsem() is broken.\n\n','cpr','err'); end;
    [v,arg]=parsem('selftest',{'asd'},@setupm);
    if(~isequal(v,1) || ~isequal(arg, {'asd'})); vprintf('ERROR: parsem() is broken.\n\n','cpr','err'); end;
    [v,arg]=parsem('tommsch',{'asd','tommsch',2},'is the best','set');
    if(~isequal(v,'is the best') || ~isequal(arg, {'asd', 'tommsch', 'is the best'})); vprintf('ERROR: parsem() is broken.\n\n','cpr','err'); end;
    [v,arg]=parsem('tommsch',{'asd','tommsch',0,'tommsch'},'is the second best','condset');
    if(~isequal(v,1) || ~isequal(arg, {'asd', 'tommsch', 1})); vprintf('ERROR: parsem() is broken.\n\n','cpr','err'); end;
    [v,arg]=parsem('tommsch',{'asd'});
    if(~isequal(v,0) || ~isequal(arg, {'asd'})); vprintf('ERROR: parsem() is broken.\n\n','cpr','err'); end;

    vprintf('plotm\n');
    figure; plotm(randn(1,40)); title('plotm: Some straight lines or points');
    figure; hold on; plotm(randn(2,40),'.-','box',2); title('plotm: Some lines and something like a square');
    figure; plotm(randn(3,40),'resolution',0,'MarkerSize',100); title('plotm: A pointcloud.');
    figure; hold on; plotm(randn(3,40),'resolution',100,'surface','contour'); view(3); title('plotm: A surface with contour lines');
    figure; plotm(num2cell(reshape(cumsum(rand(20,1)),2,[])',2)); title('plotm: Intervals.');
    figure; plotm(sequence(randn(20),[0;0]),'rotate',10); title('plotm: The surface of a sequence rotating.');
    figure; plotm(sequence(randn(20,1),[0])); title('plotm: The plot of a sequence.');
    
    vprintf('repcellm\n');
    if(~isequal(repcellm(10 ,2),{10;10})); fprintf('ERROR: repcellm() is broken.\n\n'); end;;
    if(~isequal(repcellm(10 ,[2 3]),{10 10 10;10 10 10})); fprintf('ERROR: repcellm() is broken.\n\n'); end;;
    
    vprintf('repmatm\n');
    if(~isequal(repmatm(10 ,2),[10;10])); fprintf('ERROR: repmatm() is broken.\n\n'); end;;
    if(~isequal(repmatm(10 ,[2 3]),[10 10 10;10 10 10])); fprintf('ERROR: repmatm() is broken.\n\n'); end;;
    
    vprintf('sizem\n');
    [a,b,c]=sizem(zeros(2,0,1));
    if(~isequal(a,2) || ~isequal(b,0) || ~isequal(c,1)); fprintf('ERROR: sizem() is broken.\n\n'); end;;
    a=sizem(1); if(~isequal(a,1)); fprintf('ERROR: sizem() is broken.\n\n'); end;;
    a=sizem([1;2]); if(~isequal(a,2)); fprintf('ERROR: sizem() is broken.\n\n'); end;;
    a=sizem([1 2]); if(~isequal(a,[1 2])); fprintf('ERROR: sizem() is broken.\n\n'); end;;
    a=sizem([]); if(~isequal(a,[])); fprintf('ERROR: sizem() is broken.\n\n'); end;;
    
    %sph2cartm % tested above
    %sph2cart2m % tested above
    
    vprintf('squeezem\n');
    if(~isequal(squeezem([2 1 3]),[2;1;3])); fprintf('ERROR: squeezem() is broken.\n\n'); end;;
    
    vprintf('summ\n');
    if(~isequal(summ([2 3; -4 -5],[],'abs'),14)); fprintf('ERROR: squeezem() is broken.\n\n'); end;;
    if(~isequal(summ([2 3; -4 -5]),-4)); fprintf('ERROR: squeezem() is broken.\n\n'); end;;
    if(~isequal(summ([2 3; -4 -5],1),[-2 -2])); fprintf('ERROR: squeezem() is broken.\n\n'); end;;
    
    vprintf('upsamplem\n');
    try
        [d,dmin]=upsamplem([1 2 ; 3 4], [0;0], [2 1; 0 -2],Inf);
        if(~isequal(d,[inf inf 1;2 inf inf;inf inf 3;4 inf inf]) || ~isequal(dmin,[0;-2])); fprintf('ERROR: upsamplem() is broken.\n\n'); end;
        [d,dmin]=upsamplem([],0,2);
        if(~isequal(d,[]) || ~isequal(dmin,[0])); fprintf('ERROR: upsamplem() is broken.\n\n'); end;
        [d,dmin]=upsamplem([1 2 3]',0,-1);
        if(~isequal(d,[3 2 1]') || ~isequal(dmin,-2)); fprintf('ERROR: upsamplem() is broken.\n\n'); end;        
        [d,dmin]=upsamplem([1 2 3]',0,1);
        if(~isequal(d,[1 2 3]') || ~isequal(dmin,0)); fprintf('ERROR: upsamplem() is broken.\n\n'); end;           
        [d,dmin]=upsamplem([1 2 ; 3 4], [0;0], [2 0;0 2]);
        if(~isequal(d,[1 0 2;0 0 0;3 0 4]) || ~isequal(dmin,[0;0])); fprintf('ERROR: upsamplem() is broken.\n\n'); end;        
    catch
        fprintf('ERROR: ''upsamplem'' broken. Probably tmisc package not installed.\n');
    end
    
    vprintf('zerosm\n');
    if(~isequal(zerosm(),[])); fprintf('ERROR: zerosm() is broken.\n\n'); end;
    if(~isequal(zerosm([]),[])); fprintf('ERROR: zerosm() is broken.\n\n'); end;
    if(~isequal(zerosm([1,2]),[0 0])); fprintf('ERROR: zerosm() is broken.\n\n'); end;
    if(~isequal(zerosm(3),[0;0;0])); fprintf('ERROR: zerosm() is broken.\n\n'); end;
end


function dummy; end %#ok<DEFNU> %Generates an error, if the 'end' of a function is missing. 
