function setupsequence(flag)
% setupsequence( [flag] )
% Demonstrates how to use the sequence-packge and tests if the sequence-package works correctly
% if flag is 1, then the functions in the package are tested.
%
% Written by: tommsch, 2018

%#ok<*NOPRT>
%#ok<*ASGLU>
%#ok<*NASGU>

if(nargin>=1 && flag); selftestsequence; return; end;

vprintf('Either start the function with argument <1> or read the source code of this file.\n'); 
return; 
2; %#ok<UNRCH>

end

function selftestsequence
    vprintf('The sequence-package will be tested. All output on the screen should be followed by an explanation.\n','cpr',[.1 .4 .1]);
    vprintf('If there is some error or un-commented output, the test fails.\n\n','cpr',[.1 .4 .1]);
    
    vprintf('addsequence\n');
    val=addsequence([1 2 ],[4;5],[1;1],[1;0]);
    if(~isequal(val,[4 1 2;5 0 0])); vprintf('ERROR: addsequence() is broken.\n\n','cpr','err'); end;    
    
    vprintf('characteristic\n');
    characteristic([1 2 3]','amin',[-1;0;0]);
    characteristic([1 2 3]');
    characteristic([1 2 3],'amin',[0]);
    characteristic([1 2 3]);
    [a,amin]=characteristic([1 0; 2 0; 1 -1]','amin',[-1; -1]);
    if(~isequal(a,[0 0;0 0;1 1;0 1]) || ~isequal(amin,[-1;-1])); vprintf('ERROR: characteristic() is broken.\n\n','cpr','err'); end;
    [a,amin]=characteristic([1 0; 2 0; 1 -1]','amin',[1; -1],'amax',[2;2]);
    if(~isequal(a,[1 1 0 0;0 1 0 0])); vprintf('ERROR: characteristic() is broken.\n\n','cpr','err'); end;
    
    vprintf('constructmu\n');
    mu=constructmu(2,2);
    if(~isequal(mu,[2 1 0;0 1 2])); vprintf('ERROR: constructmu() is broken.\n\n','cpr','err'); end;
    
    vprintf('diffsequence\n');
    diffsequence([1 1 1]',1);
    val=diffsequence([1 2 3; 4 5 6],[1 0],'cell');
    if(~isequal(val{1},[1 2 3; 3 3 3; -4 -5 -6])); vprintf('ERROR: diffsequence() is broken.\n\n','cpr','err'); end;    
    diffsequence([1 2 3; 4 5 6],[1 0]);
    diffsequence([1 2 3; 4 5 6],1);
    val=diffsequence([1 2 3; 4 5 6],1,'equalsize');
    if(~isequal(val,{[1 2 3 0;3 3 3 0;-4 -5 -6 0]; [1 1 1 -3;4 1 1 -6;0 0 0 0]})); vprintf('ERROR: diffsequence() is broken.\n\n','cpr','err'); end;    
    
    vprintf('equalizeminidx\n');
    val=equalizeminidx({[10 ; 10],[1 1]';[1],[0 0]'});
    if(~isequal(val{1},[0 0;0 10;0 10]) || ~isequal(val{2},[1 0;0 0;0 0])); vprintf('ERROR: equalizeminidx() is broken.\n\n','cpr','err'); end;    
    
    vprintf('mask2symbol/symbol2mask\n');
    mask=symbol2mask(mask2symbol([1 2 3; 4 5 6]));
    if(~isequal(mask,[1 2 3; 4 5 6])); vprintf('ERROR: mask2symbol()/symbol2mask() is broken.\n\n','cpr','err'); end;
    mask2symbol([1]);
    val=mask2symbol([]);
    symbol2mask(val);
    val=mask2symbol([1 1],'dim',1); syms z1;
    if(~isequal(val,1 + z1)); vprintf('ERROR: mask2symbol() is broken.\n\n','cpr','err'); end;
    val=mask2symbol([1 1],'var','x'); syms y;
    if(~isequal(val,1 + y)); vprintf('ERROR: mask2symbol() is broken.\n\n','cpr','err'); end;
    val=mask2symbol([1 1],'var',{'b','a'}); syms a b;
    if(~isequal(val,1 + a)); vprintf('ERROR: mask2symbol() is broken.\n\n','cpr','err'); end;
    val=mask2symbol([1 1],'amin',[0;1]); syms z2;
    if(~isequal(val,z2^2+z2)); vprintf('ERROR: mask2symbol() is broken.\n\n','cpr','err'); end;
    mask=symbol2mask(val,'dim',2,'sym','var','z1');
    if(~isequal(mask,sym([0 1 1]))); vprintf('ERROR: symbol2mask() is broken.\n\n','cpr','err'); end;
    
    vprintf('setidx\n');
    val=setidx([1 2; 2 3],[1 1]',[0 0]');
    val=setidx([1 2; 2 3],[1 1]',[0 0]',[2 3]');
    if(~isequal(val,[0 0 0 0;0 1 2 0;0 2 3 0])); vprintf('ERROR: setidx() is broken.\n\n','cpr','err'); end;    
    
    vprintf('supp\n');
    suppval=supp([],2,[0]);
    if(~isequal(suppval,zeros(2,0))); vprintf('ERROR: supp() is broken.\n\n','cpr','err'); end;
    suppval=supp([1 1; 0 1],2,[1;-1]);
    if(~isequal(suppval,[1 1 2;-1 0 0])); vprintf('ERROR: supp() is broken.\n\n','cpr','err'); end;
    
    %symbol2mask %tested above
    
    %setupsequence %this file
    
    vprintf('sequence\n');
    if(~isequal(ndims(sequence([1 2 3])),2)); vprintf('ERROR: sequence() is broken.\n\n','cpr','err'); end;    
    if(~isequal(ndims(sequence([1 2 3]')),1)); vprintf('ERROR: sequence() is broken.\n\n','cpr','err'); end;    
    sequence([1 2 3 4],[1]);
    vprintf('1 error message about unequal dimensions should be here.\n','cpr',[.1 .4 .1]);
    c1=sequence([-1 2 0 4],[1 1]);
    c2=sequence([10 20 30 40],[-5; 0]);
    c3=sequence([1 2 0 4],[1 0]);
    if(~isequal(c1.c,[-1 2 0 4])); vprintf('ERROR: sequence() is broken.\n\n','cpr','err'); end;    
    if(~isequal(c1.idx,[1;1])); vprintf('ERROR: sequence() is broken.\n\n','cpr','err'); end;    
    if(~isequal(ref(c1,0),0)); vprintf('ERROR: sequence() is broken.\n\n','cpr','err'); end;    
    if(~isequal(ref(-c1,1),1)); vprintf('ERROR: sequence() is broken.\n\n','cpr','err'); end;    
    2*c1; %#ok<VUNUS>
    c1/2; %#ok<VUNUS>
    c1*2; %#ok<VUNUS>
    c1+c2; %#ok<VUNUS>
    c1.*c2; %#ok<VUNUS>
    c1./c2; %#ok<VUNUS>
    c3<=c1; c3<c1; c3>c1; c3>=c1; c4=c3;  %#ok<VUNUS>
    if(~isequal(c3,c4)); vprintf('ERROR: sequence() is broken.\n\n','cpr','err'); end;
    sum(c1);
    norm(c1,3);
    supp(c1);
    c1.idxmin; 
    c1.idxmax; 
    nnz(c1);
    numel(c1); 
    ndims(c1);
    size(c1);
    [cc1,cc2]=equalize(c1,c2);
    if(~isequal(shrink(cc1),c1)); vprintf('ERROR: sequence/shrink() or sequence/equalize() is broken.\n\n','cpr','err'); end;
    norm(c1,1); norm(c1,2); norm(c1,2.5); norm(c1,inf); norm(c1,0); norm(c1,-2); norm(c1,-inf);
    figure; plot(c1,'point'); title('sequence/plot - points');
    sum(c1);
    supp(c1);
    symbol(c1);
    upsample(c1,[2 1; 0 2]);
    conv(c1,c2,c1);
    characteristic(c1);
    diffsequence(c1,[1 1]);
end


function dummy; end %#ok<DEFNU> %Generates an error, if the 'end' of a function is missing. 
