function setupsubdivision(flag)
% setupsubdivision( [flag] )
% Demonstrates how to use the subdivision and tjsr-package and tests if the subdivision package works correctly
% if flag is 1, then the functions in the subdivision-package are tested.
%
% Written by: tommsch, 2018

%#ok<*NOPRT>
%#ok<*ASGLU>
%#ok<*NASGU>

if(nargin>=1 && flag); selftestsubdivision; return; end;

vprintf('Either start the function with argument <1> or read the source code of this file.\n'); 
return; 
2; %#ok<UNRCH>

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 1-D example of multiple subdivision scheme
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%make finite set of subdivision operators
fprintf('make finite set of subdivision operators\n'); 
S1=getS('a',1/4*[1;1;3;3],'M',2);
S2=getS('a',1/4*[1;1;3;3],'M',-2);
S=[S1;S2]; %each subdivision operator is one line in the cell array. Due to strange matlab-behaviour, if the cell C shall be displayed, an error is thrown.

%plot blf with two different ordering
blf({[2 2],[1 1 2]},S); %ordering 2211211212112
blf(S); %random ordering

[T, Om]=transitionmatrix(S); %compute transition matrices and the set Omega

plotm(Om,'x') %plot the set Omega

V0=constructV(Om,0)                   %Construct space orthogonal to polynomial sequences
V0bar=constructVt(Om,0);            %Construct space of differences of delta
if(size(V0,2)~=size(V0bar,2)); 
    fprintf('Set Omega is wrong.');   %Test ifV0==V0bar, if not the JSR is no characterization for regularity
end
TV0=restrictmatrix(T,V0); vdisp(TV0); %restrict transition matrices

[JSR, type]=tjsr(TV0);  %compute JSR of restricted matrices. If the algorithm does not terminate, set delta to some value less then one.
%[JSR, type]=tjsr(TV0,'maxsmpdepth',2,'delta',.999999); %compute the continuity regularity. 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% s-D example of subdivision scheme
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

S=getS('2_butterfly','sym');        %Butterfly-scheme
[~,~,xyzv]=blf(S,'iteration',3);   %plot the blf
tile(getS(S,'supp'));               %plot the support of the blf
[T, Om]=transitionmatrix(S);        %compute transition matrices and the set Omega
plotm(Om,'x');                      %plot the set Omega
V3=constructV(Om,3);                %Construct space of fourth differences
[~,TV3]=restrictmatrix(T,V3);           %restrict transition matrices

% The transition matrices of this scheme have a bunch of invariant subspaces. 
% Thus the invariant-polytope algorithm behaves badly.
% To encounter that problem, we attack the subspaces one by one
TV3=invariantsubspace(TV3)
TV3=[invariantsubspace(TV3{1}) invariantsubspace(TV3{2})]

%lets have a look at the subspaces sizes
cellfun(@(x) size(x{1},2),TV3)

[JSR1, type1]=tjsr(TV3{1}); %compute Hoelder regularity.
[JSR2, type2]=tjsr(TV3{2},'maxsmpdepth',1,'delta',.999999999); %compute Hoelder regularity.
[JSR3, type3]=tjsr(TV3{3},'maxsmpdepth',1,'delta',.999999999); %compute Hoelder regularity.
cellfun(@(x) max(abs([x{:}])),TV3(4:end)) %all the other spectral radii

end

function selftestsubdivision
    vprintf('The subdivision-package will be tested. All output on the screen should be followed by an explanation.\n','cpr',[.1 .4 .1]);
    vprintf('If there is some error or un-commented output, the test fails.\n\n','cpr',[.1 .4 .1]);

    vprintf('getS\n');
    S1=getS('a',[1 2 1],'M',2);
    vprintf('Message that the mask does not sum up should be here.\n','cpr',[.1 .4 .1]);
    vprintf('Message that the mask is transposed should be here.\n','cpr',[.1 .4 .1]);
    getS('a',1/2*[1 1 2]','M',2,'D',[0 2],'bigcheck');
    vprintf('2 error messages should be here.\n','cpr',[.1 .4 .1]);
    S1=getS('a',[1 2 1],'M',2,'name','nocheck');
    S1=getS('a',1/2*[1 2 1]','M',2);
    if(~isequal(S1,getS(S1))); vprintf('ERROR: getS() is broken.\n\n','cpr','err'); end;
    if(~isequal(S1,getS({1/2*[1 2 1]',2}))); vprintf('ERROR: getS() is broken.\n\n','cpr','err'); end;
    S1=getS('1_121');
    getS(S1,'supp');
    getS([S1; S1],'supp');
    getS(S1,'OmegaRR');
    getS(S1,'characteristic');
    getS('1_rand');
    getS('2_rand');
    getS('1_all','nocheck'); %1_all also includes non-stationary schemes
    S2=getS('2_rand');
    S=getS([S1;S2]);
    vprintf('Message that some dimension or size is wrong.\n','cpr',[.1 .4 .1]);
    S2=getS('1_1133');
    S=getS([S1;S2]);
    try;
        getS(1,'nocheck'); 
    catch
        vprintf('An error in the line directly above is not severe.\n','cpr','green');
    end
    val=getS('1_143'); if(norm(val{1}.c-1/4*[1;4;3])>10*eps); vprintf('ERROR: getS() is broken.\n\n','cpr','err'); end;
    val=getS('3_rand'); val=getS(val);
    val=getS('a',1/2*[1 2 1]','M',2); if(~isequal(val{3},[0 1])); vprintf('ERROR: getS() is broken.\n\n','cpr','err'); end;
    getS(S1);
    
    vprintf('transitionmatrix\n');
    [T,Om]=transitionmatrix(S);
    [T,Om]=transitionmatrix(S,'colsum',1);
    if(~isequal(Om,[0 1 2])); vprintf('ERROR: transitionmatrix() is broken.\n\n','cpr','err'); end;
    T=transitionmatrix(S,'Omega',[0;0]);
    vprintf('Message that some dimension or size is wrong.\n','cpr',[.1 .4 .1]);
    [T,Om]=transitionmatrix(S,'Omega',[50],'colsum',1);
    vprintf('4 error messages should be here.\n','cpr',[.1 .4 .1]);
    if(~isequal(Om,50)); vprintf('ERROR: transitionmatrix() is broken.\n\n','cpr','err'); end;
    T=transitionmatrix(S,'noflat');
    if(~iscell(T{1})); vprintf('ERROR: transitionmatrix() is broken.\n\n','cpr','err'); end;
    [T,Om,V0]=transitionmatrix(S,'V',0);
    if(~isequal(null(V0','r'),[1;1;1])); vprintf('ERROR: transitionmatrix() is broken.\n\n','cpr','err'); end;
    [T,Om,V1]=transitionmatrix(S,'V',3);
    if(~isequal(Om,[0 1 2 3 4])); vprintf('ERROR: transitionmatrix() is broken.\n\n','cpr','err'); end;
    T=transitionmatrix(S,'noflat','colsum',1);
    T=transitionmatrix(S,'onlyindex');
    T=transitionmatrix(S);
    
    
    %from here on the functions are called in lexicographic ordering
    
    vprintf('blf\n');
    figure; blf(S,'iteration',4,'verbose',0); 
    title('blf -- There should be some 1d function');
    figure; blf(getS('2_butterfly'),'iteration',3,'diff',1,'verbose',0); 
    title('blf -- There should be two 2d functions');
    blf(S,'plot',0,'verbose',0);
    figure; [c,PM,xyzv,oo]=blf({[],1},S,'start',[1 -1]','removezero',1,'start',[0 0 0 1 0 0 0]','iteration',5,'verbose',0);
    title('blf -- There should be a hat function.');
    if(~isequal(PM,32)); vprintf('ERROR: blf() is broken.\n\n','cpr','err'); end;
    if(~isequal(oo,{[],[1]})); vprintf('ERROR: blf() may be broken.\n\n','cpr','err'); end;
    if(~isequal(size(xyzv,1),2)); vprintf('ERROR: blf() may be broken.\n\n','cpr','err'); end;
    try; 
        Ssym=getS('1_121'); Ssym{1}.c=sym(Ssym{1}.c);
        figure; blf(Ssym,'iteration',4,'verbose',0);
        title('blf -- There should be nearly a hat function.');
    catch; 
        vprintf('ERROR: blf() may be broken.\n\n','cpr','err');
    end
    
    
    
    vprintf('checktile\n');
    M1=[1 2; -2 -2];  D1=[0 1; 0 -1]; 
    M2=[-1 1; -1 -2]; D2=[0 0 0; -2 -1 0]; 
    M12=M2*M1; D12=setplus(M2*D1,D2);
    [f1,C1,T1]=checktile(M1,D1,'verbose',0,'legacy');
    if(~isequal(size(C1),[6 6])); vprintf('ERROR: checktile() is broken: C1=%i, C2=%i, C3=%i\n\n',f1,f2,f12,'cpr','err'); end;
    if(~isequal(size(T1),[2 6])); vprintf('ERROR: checktile() is broken: C1=%i, C2=%i, C3=%i\n\n',f1,f2,f12,'cpr','err'); end;
    f1=checktile(M1,D1,'verbose',0);
    f2=checktile(M2,D1,'verbose',0);
    f12=checktile(M12,D12,'verbose',0);
    if(~isequal([f1 f2 f12],[true, true, false]) ); vprintf('ERROR: checktile() is broken: C1=%i, C2=%i, C3=%i\n\n',f1,f2,f12,'cpr','err'); end;
    
    
    vprintf('compresscoordinates\n');
    cc=round(3*compresscoordinates([10 20 ; 40 50],[2 1; -1 1]));
    if(~isequal(cc,[0 1 -1 0;0 1 2 3;30 120 60 150])); vprintf('ERROR: compresscoordinates() is broken.\n\n','cpr','err'); end;
    
    vprintf('constructdigit\n');    
    if(~isequal(constructdigit(2),[0 1])); vprintf('ERROR: constructdigit() is broken.\n\n','cpr','err'); end;
    d=constructdigit( [2 0; 1 2], 'ZZ',[0 0; 1 0]','sym');
    d=constructdigit( [2 0; 1 2], 'ZZ',[0 0; 1 0]','classify','sym','verbose',0);
    d=constructdigit( [2 0; 1 2], 'random');
    if(~isequal(size(d),[2,4])); vprintf('ERROR: constructdigit() is broken.\n\n','cpr','err'); end;
    d=constructdigit( [2 0; 1 2] );
    if(~isequal(d,[0 0 1 1;0 1 1 2])); vprintf('ERROR: constructdigit() is broken.\n\n','cpr','err'); end;
    
    vprintf('constructOmega\n');
    S=getS('1_1331');
    Om=constructOmega(S,'legacy');
    if(~isequal(sort(Om),[0 1 2])); vprintf('ERROR: constructOmega() is broken.\n\n','cpr','err'); end;
    Om=constructOmega(S,'Omega',[4 3 2 1 0],'stable');
    if(~isequal(Om,[4 3 2 1 0])); vprintf('ERROR: constructOmega() is broken.\n\n','cpr','err'); end;
    figure; Om=constructOmega(getS('2_butterfly'),'plot');
    title('constructOmega - Set of points');
    constructOmega(S);
    
    
    vprintf('constructordering\n');
    oo=constructordering([1 2],[3 ],[10 20],[30]);
    if(~isequal(oo,{[1 2;10 20], [3;30]})); vprintf('ERROR: constructordering() is broken.\n\n','cpr','err'); end;
    
    
    %constructPhi %untested
    
    %constructU   %untested
    
    vprintf('constructV\n');
    Om=[0 1 2 3];
    V0=constructV([1 1 1;0 1 1],'01');
    if(~iscell(V0)); vprintf('ERROR: constructV() is broken.\n\n','cpr','err'); end;
    V0=constructV(Om,0);
    V5=constructV(Om,5);
    if(~isempty(V5)); vprintf('ERROR: constructV() is broken.\n\n','cpr','err'); end;
    
    vprintf('constructVt\n');
    Om=[0 1 2 3];    
    Vt0=constructVt([1 1 1;0 1 1],'01');
    Vt=constructVt(Om);
    if(~iscell(Vt0)); vprintf('ERROR: constructVt() is broken.\n\n','cpr','err'); end;
    [Vt2,Om]=constructVt([1 1 0 1 1],2,'01');
    if(~isempty(Vt2)); vprintf('ERROR: constructVt() is broken.\n\n','cpr','err'); end;
    if(~isequal(Om,[0 0 0 0;0 1 3 4])); vprintf('ERROR: constructVt() is broken.\n\n','cpr','err'); end;
    
    vprintf('daubechiesmask\n');
    a=daubechiesmask( 5 );
    if(~issym(a)); vprintf('ERROR: daubechiesmask() may be broken.\n\n','cpr','err'); end;
    
    %differencescheme %untested
    
    vprintf('dimVVt\n');
    dim=dimVVt([1 1 0 0;1 0 1 1;0 1 0 0],'01','verbose',0);
    if(~isequal(dim,[5 3; 3 0])); vprintf('ERROR: constructVt()/constructV()/dimVVt() is broken.\n\n','cpr','err'); end;
    dim=dimVVt(Om,'V','verbose',0);
    dimt=dimVVt(Om,'Vt','verbose',0);
    if(~isequal(dimt,2)); vprintf('ERROR: constructVt()/constructV()/dimVVt() is broken.\n\n','cpr','err'); end;
    if(~isequal(dim,[3 2 1])); vprintf('ERROR: constructVt()/constructV()/dimVVt() is broken.\n\n','cpr','err'); end;
    
    vprintf('findperiod\n');
    [of]=findperiod([1 2 1 2 1 2; 4 5 5 5 5 5]);
    [oo,pp,nrep]=findperiod([1 2 1 2 1 2; 4 5 5 5 5 5]);
    if(~isequal(of{1},[1;4]) || ~isequal(of{2},[2 1; 5 5])); vprintf('ERROR: findperiod() is broken.\n\n','cpr','err'); end;
    if(~isequal(oo,[1;4]) || ~isequal(pp,[2 1; 5 5])); vprintf('ERROR: findperiod() is broken.\n\n','cpr','err'); end;
    try; if(nrep<1 || nrep>3); error; end; catch; vprintf('ERROR: findperiod() is broken.\n\n','cpr','err'); end; %#ok<LTARG>
    [of]=findperiod([1 2 1 2 3 2 1 2 1 2],'fuzzy');
    if(~isempty(of{1}) || ~isequal(of{2},[1 2])); vprintf('ERROR: findperiod() is broken.\n\n','cpr','err'); end;
    [of]=findperiod([1 2 1 2 1 2 1 2 2 2 ],'everywhere','verbose',0);
    if(~isempty(of{1}) || ~isequal(of{2},[1 2])); vprintf('ERROR: findperiod() is broken.\n\n','cpr','err'); end;    
    
    
    %vprintf('getS\n'); %tested above

    
    vprintf('isordering\n');
    [of]=findperiod([1 2 1 2 1 2; 4 5 5 5 5 5]);
    if(isordering(of)~=true || isordering([2 3])~=false || isordering([])~=false); vprintf('ERROR: isordering() is broken.\n\n','cpr','err'); end;
    
    vprintf('isS\n');
    if(isS(S1)~=true || isS([2 3]~=false)); vprintf('ERROR: isS() is broken.\n\n','cpr','err'); end;    
    
    vprintf('isT\n');
    if(isT(T)~=true || isS([2 3]~=false)); vprintf('ERROR: isT() is broken.\n\n','cpr','err'); end;    
        
    vprintf('multiplyS\n');
    S1=getS('1_143'); S2=getS('1_1133'); 
    multiplyS(S1,S2); 
    S=multiplyS(S1,S2,S2);
    
    S=getS({[1 1 1]',2},'nocheck');
    vprintf('normalizeS\n');
    normalizeS({[1 0 1],2});
    vprintf('Error message should be here.\n','cpr',[.1 .4 .1]);
    Sn=normalizeS(S,'scale');
    Sn=normalizeS(S,'gauss');
    Sn=normalizeS(S,'equal');
    if(~isequal(Sn{1,1}.c,1/2*[1 2 1]')); vprintf('ERROR: normalizeS() is broken.\n\n','cpr','err'); end;    
    
    vprintf('num2ordering\n');
    figure;
    Snum=getS({[],[1 2; -1 2]});
    pt=[0.1875;0.375];
    num=ordering2num(num2ordering(Snum,pt,'l',50,'verbose',0,'plot',1),Snum);
    if(abs(num-pt)>10^-12); vprintf('ERROR: ordering2num()/num2ordering() is broken.\n\n','cpr','err'); end;
    title('num2ordering: Attractor with path inside');
    num2ordering(Snum,[1 2],'check');
    vprintf('Message that some point is outside of something should be here.\n','cpr',[.1 .4 .1]);
    [do,doint,num,proofnum]=num2ordering(Snum,pt,'check','proof','verbose',0);
    if(~isequal(do,{[1 1;2 3],[1 1;1 2]})); vprintf('ERROR: ordering2num()/num2ordering() is broken.\n\n','cpr','err'); end;
    if(~isequal(proofnum,num)); vprintf('ERROR: ordering2num()/num2ordering() is broken.\n\n','cpr','err'); end;
    Sn={[],2,[0 1]; [],2,[0 3]};
    figure;
    try
    ordering2num(num2ordering({[],[1 2]},Sn,5/3,'plot',1,'verbose',4,'l',10,'delta',.7,'round',0),Sn);
    title('num2ordering: Attractor with path inside');
    catch
    vprintf('Some error messages here are not crucial.\n','cpr',[.1 .4 .1]);
    end
    vprintf('A lot of text output may be here.\n','cpr',[.1 .4 .1]);
    
    vprintf('ordering2num\n');
    num=ordering2num({[2 1 ],[]},getS({[],2}));
    if(~isequal(num,1/2)); vprintf('ERROR: ordering2num() is broken.\n\n','cpr','err'); end;
    num=ordering2num({[2 1],[1 3]},getS({[],[1 2; -1 1]}));
    num=ordering2num({[],[1 2]},getS({[],2}));
    if(~isequal(num,1/3)); vprintf('ERROR: ordering2num() is broken.\n\n','cpr','err'); end;
    num=ordering2num({[],[]},getS({[],2}));
    if(~isequal(num,0)); vprintf('ERROR: ordering2num() is broken.\n\n','cpr','err'); end;
    
    vprintf('ordering2vector\n');
    vec=ordering2vector({ [10 20 30] , [1 2] },10);
    if(~isequal(vec,[10 20 30 1 2 1 2 1 2 1])); vprintf('ERROR: ordering2vector() is broken.\n\n','cpr','err'); end;
    ordering2vector({[],[1]},5)
    vec=ordering2vector({[1 2 3 4 5],[]},10);
    if(~isequal(vec,[1 2 3 4 5])); vprintf('ERROR: ordering2vector() is broken.\n\n','cpr','err'); end;
    ordering2vector({[],[]});
    
    vprintf('peter\n');
    peter(rand(2,30),4);
    peter(rand(2,30),.4);
    
    vprintf('restrictmatrix\n');
    S=getS('1_121');
    [T,Om,V0]=transitionmatrix(S,'V',0,'Omega',[0 1 2 3]);
    restrictmatrix(T{1},V0);
    TV1=restrictmatrix(T,[1 -3 3 1]');
    vprintf('2 Message that matrices are not invariant should be here.\n','cpr',[.1 .4 .1]);
    [TA,TT,TR,NULL,BASIS]=restrictmatrix(T,[1 -1 0 0; 1 0 -1 0; 1 -1 0 0]','smallsize');
    
    %testsubdivision %this file
    
    vprintf('tile\n');
    figure; 
    S=getS('2_butterfly');
    [Q,oo]=tile(S,'verbose',0,'peter',1000,'round',[1e-1 1e-5]); 
    if(~isequal(size(Q,1),2)); vprintf('ERROR: tile() is broken.\n\n','cpr','err'); end;
    if(any(diff(oo))); vprintf('ERROR: tile() is broken.\n\n','cpr','err'); end;
    Q=tile(S,'verbose',0,'peter',[.4 .8]); 
    try; [Q,oo]=tile(S,'verbose',0,'plot',{'MarkerSize',10}); 
    catch; vprintf('ERROR: tile() or plotm() is broken.\n\n','cpr','err'); 
    end;
    tile(S,'verbose',0,'digit'); 
    tile(S,'verbose',0,'diffdigit','iteration',3); 
    tile(S,'verbose',0,'supp'); 
    tile(S,'verbose',0,'OmegaRR','iteration',4); 
    tile({[1 2 2 1],[]},[getS('2_rand');S],'verbose',0,'iteration',5); 
    vprintf('Some warning message may be here.\n','cpr',[.1 .4 .1]);
    title('tile - Points in the plane');
    figure; tile([getS('2_rand');getS('2_rand')],'supertile',6,'round',[1e-2 1e-4],'verbose',0); 
    figure; tile([getS('2_rand'); getS('2_rand')],'supertile',6,'iteration',8,'verbose',0); 
    title('tile - supertile (something Rohrschach-like)');
    figure; tile({[],3,[0 1 5]},'interval','start',{[0 1]},'round',0)
    
    vprintf('tilearea\n');
    tilearea(tile(S,'plot',0,'verbose',0),'verbose',0); 
    
    vprintf('Some warning message may be here.\n','cpr',[.1 .4 .1]);
    %vprintf('Some text output should state something.\n\n','cpr',[.1 .4 .1]);
    
    %transitionmatrix %tested above
    
    vprintf('vector2ordering\n');
    [o1, o2]=vector2ordering([1 2 1 2 1 2; 4 5 5 5 5 5]);
    if(~isequal(o1,[1;4]) || ~isequal(o2,[2 1; 5 5])); vprintf('ERROR: vector2ordering()/findperiod() is broken.\n\n','cpr','err'); end;
end


function dummy; end %#ok<DEFNU> %Generates an error, if the 'end' of a function is missing. 
