function setuptjsr(flag)
% setuptjsr( [flag] )
% Demonstrates how to use the tjsr-packge and tests if the tjsr-package works correctly
% if flag is 1, then the functions in the package are tested.
%
% See also: testsubdivision, tjsr
%
% Written by: tommsch, 2018

%#ok<*NOPRT>
%#ok<*ASGLU>
%#ok<*NASGU>

if(nargin>=1 && flag); selftesttjsr(); return; end;

vprintf('Either start the function with argument <1> or read the source code of this file.\n'); 
return; 
2; %#ok<UNRCH>

%To see how the tjsr package works, see also testsubdivision

end

function selftesttjsr
    verbose=0;
    
    vprintf('The tjsr-package will be tested.\n','cpr',[.1 .4 .1]);
    vprintf('If there is some error or un-commented output, the test fails.\n\n','cpr',[.1 .4 .1]);
    
    vprintf('binarymatrix\n');
    val=binarymatrix(2, 2, 2^5+2^3+2^2+2^1);
    if(~isequal(val,{[0 1; 0 0],[1 1; 1 0]})); vprintf('ERROR: binarymatrix() is broken.\n\n','cpr','err'); end;
    val=binarymatrix(1, 1, 2);
    if(~isempty(val)); vprintf('ERROR: binarymatrix() is broken.\n\n','cpr','err'); end;
    if(~isempty(binarymatrix(3, 1, 1+16+32+128+256,1))); vprintf('ERROR: binarymatrix() is broken.\n\n','cpr','err'); end;
    if(~isequal(binarymatrix(3, 1, 1+16+32+128+256,0),{[1 1 0;1 1 0;0 0 1]})); vprintf('ERROR: binarymatrix() is broken.\n\n','cpr','err'); end;
    if(isempty(binarymatrix(3, 1, 1+2+8+16+256,0))); vprintf('ERROR: binarymatrix() is broken.\n\n','cpr','err'); end;
    if(~isempty(binarymatrix(2, 1, 4,1))); vprintf('ERROR: binarymatrix() is broken.\n\n','cpr','err'); end;
    if(isempty(binarymatrix(2, 1, 2,1))); vprintf('ERROR: binarymatrix() is broken.\n\n','cpr','err'); end;
    
    vprintf('blockjsr\n');
    if(~isequal(blockjsr([1 0],[.5 2]),[.5 2])); vprintf('ERROR: blockjsr() is broken.\n\n','cpr','err'); end;
    if(~isequal(blockjsr(1),1)); vprintf('ERROR: blockjsr() is broken.\n\n','cpr','err'); end;
    if(~isequal(blockjsr(0,1),1)); vprintf('ERROR: blockjsr() is broken.\n\n','cpr','err'); end;
    if(~isequal(blockjsr([0 1],1),1)); vprintf('ERROR: blockjsr() is broken.\n\n','cpr','err'); end;
    
    vprintf('chooseval\n');
    if(~islogical(chooseval(randn(1,100),[10 50]))); vprintf('ERROR: chooseval() is broken.\n\n','cpr','err'); end;
    if(~isequal([1 0 0],chooseval([inf 0 nan],1))); vprintf('ERROR: chooseval() is broken.\n\n','cpr','err'); end;
    if(~islogical(chooseval(randn(1,100),[10 50]))); vprintf('ERROR: chooseval() is broken.\n\n','cpr','err'); end;
    if(~isequal(chooseval([0 0 0 0 3 3]'),[0 0 0 0 1 1]')); vprintf('ERROR: chooseval() is broken.\n\n','cpr','err'); end;
    
    vprintf('codecapacity\n');
    figure;
    try
        codecapacity({[1 1 0],[1 -1 0]},'verbose',verbose,'plot',1); title('codecapacity');
        C=codecapacity({[1 1],[-1 -1]},'verbose',verbose);
    if(~isequal(C,{[0 1; 1 1],[1 1; 1 0]})); vprintf('ERROR: codecapacity() may be broken.\n\n','cpr','err'); end;
    catch
        vprintf('ERROR: codecapacity() is broken.\n\n','cpr','err');
    end
    
    vprintf('computepolytopenorm\n');
    val=computepolytopenorm([0 3]', [], 1, 0, 0);
    if(~isequal(val,inf)); vprintf('ERROR: computepolytopenorm() is broken.\n\n','cpr','err'); end;    
    val=computepolytopenorm([], [1 2; 2 1], 1, 0, 0);
    if(~isempty(val)); vprintf('ERROR: computepolytopenorm() is broken.\n\n','cpr','err'); end;        
    val=computepolytopenorm([0 3; 2 2]', [1 2; 2 1]', 1, 0, 0);
    if(norm(val-[3 4/3])>10e-12); vprintf('ERROR: computepolytopenorm() is broken.\n\n','cpr','err'); end;    
    val=computepolytopenorm([0 3; 2 2]', [1 2; 2 1]', 0, 0, 0);
    if(norm(val-[3/2 4/3])>10e-12); vprintf('ERROR: computepolytopenorm() is broken.\n\n','cpr','err'); end;        
    
    try
        vprintf('daubechiesmatrix\n');
        D21=daubechiesmatrix(2,'gugl','verbose',verbose); val1=max(cellfun(@(x) max(abs(eig(double(x)))),D21));
        D22=daubechiesmatrix(2,'jung','verbose',verbose); val2=max(cellfun(@(x) max(abs(eig(double(x)))),D22));
        if(norm(val1-val2)>10e-10); vprintf('ERROR: daubechiesmatrix() is broken.\n\n','cpr','err'); end;
    catch
        vprintf('ERROR: daubechiesmatrix() is broken. Probably the subdivision-package is not installed.\n\n','cpr','err');
    end
    %vprintf('There should be text output.\n\n','cpr',[.1 .4 .1]);
    
    vprintf('estimatejsr\n');
    TT={[1 2; 0 1],[1 2; 1 0]};
    try
        val=estimatejsr(TT);
        if(val(1)-10e-9>2 || val(2)+10e-9<2); 
            
            vprintf('ERROR: estimatejsr() is broken.\n\n','cpr','err');
        end
    catch
        vprintf('ERROR: estimatejsr() is broken.\n\n','cpr','err');
    end
    %vprintf('There should be two numbers text output.\n','cpr',[.1 .4 .1]);
    
    try
        vprintf('estimatepolytopenorm\n');
        VV=randn(3,10); pp=randn(3,10);
        [normval1,bd1]=estimatepolytopenorm(VV, VV, [], 11);
        [normval2,bd2]=estimatepolytopenorm(pp, pp, [], 1);
    catch
        vprintf('ERROR: estimatepolytopenorm() is broken.\n\n','cpr','err');
    end

    vprintf('findsmp:.....\n        ');
    AA={[1 -1; 3 -2], [1 3; -1 -1]};
    vprintf('|'); 
    findsmp(AA,'bruteforce','minsmpdepth',3,'verbose',verbose,'minJSR',3,'searchonlyonecandidate');
    findsmp(AA,'bruteforce','minsmpdepth',7,'maxsmpdepth',8,'verbose',verbose);
    findsmp(AA,'gripenberg','maxsmpdepth',5,'verbose',verbose);
    findsmp(AA,'genetic','maxsmpdepth',5,'verbose',-1);
    [c4,nc,info]=findsmp(AA,'maxsmpdepth',0,'verbose',verbose,'searchonlyonecandidate','minJSR',1.5);
    if(~isequal(c4{1}, [1;2]) && ~isequal(c4{1}, [2;1])); vprintf('ERROR: findsmp() broken.\n\n','cpr','err'); end;    
    %
    AA={[1 -1; 3 -2], [1 3; -1 -1]};
    vprintf('|'); 
    findsmp(AA,'select',0,'maxsmpdepth',5,'verbose',verbose);
    findsmp(AA,'select',0,'maxsmpdepth',5,'verbose',verbose,'vpa');
    findsmp(AA,'select',1,'maxsmpdepth',5,'verbose',verbose);
    findsmp(AA,'select',1,'maxsmpdepth',5,'verbose',verbose,'vpa');
    findsmp(AA,'select',2,'maxsmpdepth',5,'verbose',verbose);
    findsmp(AA,'select',2,'maxsmpdepth',5,'verbose',verbose,'vpa',2);
    %
    AA=tgallery('rand_gauss',100,2,'sparse',.8,'rho');
    vprintf('|'); 
    findsmp(AA,'maxsmpdepth',10,'verbose',verbose);
    findsmp(AA,'maxsmpdepth',10,'verbose',verbose,'sparse',1);
    %
    AA=tgallery('rand_gauss',30,2,'seed',102,'rho');    
    vprintf('|'); 
    findsmp(AA,'maxsmpdepth',20,'verbose',verbose,'nearlycandidate',.1);
    %
    AA=tgallery('rand_gauss',10,100,'seed',102,'rho');    
    vprintf('|'); 
    findsmp(AA,'select',0,'maxsmpdepth',20,'verbose',verbose,'N',2);
    findsmp(AA,'select',1,'maxsmpdepth',20,'verbose',verbose,'N',2);
    findsmp(AA,'select',2,'maxsmpdepth',20,'verbose',verbose,'N',2);
    %
    AA=tgallery('rand_gauss',10,3,'seed',102,'rho');    
    findsmp(AA,'select',2,'maxsmpdepth',5,'verbose',verbose,'N',inf);
    %
    vprintf('\n');
    
    vprintf('intersectinterval\n');
    if(~isequal(intersectinterval([4 5],[1 4],[-inf 4]),[4 4])); vprintf('ERROR: intersectinterval() broken.\n\n','cpr','err'); end;    
    if(~isequal(intersectinterval(),[-inf inf])); vprintf('ERROR: intersectinterval() broken.\n\n','cpr','err'); end;        
    if(~isequal(intersectinterval([0 0]),[0 0])); vprintf('ERROR: intersectinterval() broken.\n\n','cpr','err'); end;        
    if(~isequal(intersectinterval([0 0],[1 1]),[])); vprintf('ERROR: intersectinterval() broken.\n\n','cpr','err'); end;        
    if(~isequal(intersectinterval([-1 0],[1 2],'failsafe','minmax'),[-1 2])); vprintf('ERROR: intersectinterval() broken.\n\n','cpr','err'); end;        
    if(~isequal(intersectinterval([-1 0],[1 2],'failsafe','maxmin'),[0 1]));  vprintf('ERROR: intersectinterval() broken.\n\n','cpr','err'); end;        
    
    vprintf('invariantsubspace\n');
    [M,B]=invariantsubspace({[1 1 1; 1 0 1; 0 1 0], [1 -1 0; 0 2 0; 1 1 2]},'verbose',verbose);
    if(size(M,2)~=3); vprintf('ERROR: invariantsubspace() broken.\n\n','cpr','err'); end;    
    [M,B]=invariantsubspace({[1 1 1; 1 0 1; 0 1 0], [1 -1 0; 0 2 0; 1 1 2]},'verbose',verbose,'none');
    if(~isequal(B,eye(3))); vprintf('ERROR: invariantsubspace() broken.\n\n','cpr','err'); end;        
    
    vprintf('leadingeigenvector\n');
    MM=[1 2;1 1];
    [M,v0,v0s,mult]=leadingeigenvector(MM,'verbose',verbose);
    if(~isequal(M{1},MM) || norm(v0{1}-1/sqrt(3)*[sqrt(2);1])>10e-12 || norm(v0s{1}-[1;sqrt(2)]/dot(v0{1},[1;sqrt(2)]))>10e-12 || ~isequal(mult,1));  
        vprintf('ERROR: leadingeigenvector() is broken.\n\n','cpr','err'); 
    end;
    MM=[1 2;-2 1];
    [M,v0,v0s,mult]=leadingeigenvector(MM,'verbose',verbose,'complexeigenvector',0);
    if(~isequal(numel(M),2));
        vprintf('ERROR: leadingeigenvector() is broken.\n\n','cpr','err'); 
    end;
    MM=[1 2 0;-2 1 0; 0 0 sqrt(5)];
    [M,v0,v0s,mult]=leadingeigenvector(MM,'verbose',verbose,'complexeigenvector',1);
    if(~isequal(M{1},MM) || ~isequal(v0{1},[0;0;1]) || ~isequal(v0s{1},[0;0;1]) || ~isequal(mult,1));  
        vprintf('ERROR: leadingeigenvector() is broken.\n\n','cpr','err'); 
    end;
    MM={[0 1;-1 0], [1 0;0 1]};
    [M,v0,v0s,mult]=leadingeigenvector(MM,'verbose',verbose,'complexeigenvector',1);
    if(~isequal(numel(M),4));  
        vprintf('ERROR: leadingeigenvector() is broken.\n\n','cpr','err'); 
    end;
    [M,v0,v0s,mult]=leadingeigenvector(MM,'verbose',verbose,'complexeigenvector',2);
    if(~isequal(numel(M),2) || ~isequal(mult,[2 2]));  
        vprintf('ERROR: leadingeigenvector() is broken.\n\n','cpr','err'); 
    end;
    %'complexeigenvector',3 %XX not yet implemented
    TT={[1 2 ; 1 1],[0 0; 1 0]}; oo={[1 2]',[1]'};
    [M,v0,v0s,mult,oo]=leadingeigenvector(TT,oo,'verbose',verbose);
    
    %this tests makeorderinggraph
    vprintf('makeorderinggraph\n');
    figure;
    [~,~,finfo]=findsmp(tgallery('rand_gauss',2,2,'seed',10,'rho'),'maxsmpdepth',10,'plot','tree','N',2,'verbose',verbose);
    title('findsmp: Computed orderings');
    if(~isfield(finfo,'G'));  vprintf('ERROR: findsmp() is broken.\n\n','cpr','err'); end;    
    figure;
    oo=[1 1 1 2 1 2 2 2 1 1;0 2 1 0 2 1 2 1 1 2;0 0 0 0 1 0 0 1 2 1]; makeorderinggraph(oo,'lableedge',0,'verbose',verbose);
    title('makeorderinggraph');
    
    vprintf('makepositive\n');
    if(~isequal(makepositive([ 0 -1 2]), [0 1 -2])); vprintf('ERROR: makepositive() is broken.\n\n','cpr','err'); end;    
    if(norm(makepositive(1i)-1)>10e-12); vprintf('ERROR: makepositive() is broken.\n\n','cpr','err'); end;    
    
    vprintf('partitionatepolytope\n');
    figure;
    [chain, MST]=partitionatepolytope(randn(2,100),2,1); 
    vprintf('Some text may be printed.\n','cpr',[0 .6 0]);
    title('partitionatepolytope')

    vprintf('preprocessmatrix\n');
    preprocessmatrix({[-1 2; 2 3],[-1 2; 2 3]},'verbose',verbose);
    if(~isequal(preprocessmatrix({-1},'verbose',verbose),{1})); vprintf('ERROR: preprocessmatrix() is broken.\n\n','cpr','err'); end;  
    preprocessmatrix({[-1 2; 2 3],[-1 2; 2 3]},'verbose',verbose,'inverse',1,'addinverse',1,'transpose',1,'addtranspose',1,'makepositive',0,'timestep',.9,'perturbate',.001,'removezero',0,'removeduplicate',0,'basechange','random','exponential',1);
    preprocessmatrix({[-1 2; 2 3],[-1 2; 2 3]},'verbose',verbose,'basechange',1);
    preprocessmatrix({[-1 2; 2 3],[-1 2; 2 3]},'verbose',verbose,'basechange',[1 1;1 -1]);
    
    vprintf('reducelength\n');
    val=reducelength({[2 1 2 2 1 2],[3 1]});
    if(~isequal(val,{[1 2 2],[1 3]})); vprintf('ERROR: reducelength() broken.\n\n','cpr','err'); end;    
    val=reducelength({[2 1 2 2 1 2],[3 1]},0);
    if(~isequal(val,{[1 2 2],[1 3]})); vprintf('ERROR: reducelength() broken.\n\n','cpr','err'); end;        
    val=reducelength([2 1 2 2 1 2],1);
    if(~isequal(val,[2 1 2])); vprintf('ERROR: reducelength() broken.\n\n','cpr','err'); end;            
    val=reducelength([2 1 2 2 1 2],2);
    if(~isequal(val,[1 2 2 1 2 2])); vprintf('ERROR: reducelength() broken.\n\n','cpr','err'); end;                
    
    vprintf('removecombination\n');
    val=removecombination({[1 1 2],[1]});
    if(size(val,2)~=2 || ~searchincellarray(2,val,1) || ~searchincellarray(1,val,1)); vprintf('ERROR: removecombination() is broken.\n\n','cpr','err'); end;    
    val=removecombination({[1 1 2],[1]},'add');
    if(~isequal(val,{[1 1 2],[1],[1 2],[2]})); vprintf('ERROR: addcombination() may be broken.\n\n','cpr','err'); end;    
   
    vprintf('tbuildproduct_fast\n');
    val=tbuildproduct_fast({2 4 5},[1 0 3]);
    if(~isequal(val,10)); vprintf('ERROR: tbuildproduct_fast() is broken.\n\n','cpr','err'); end;
    vv=[1;1];
    val1=tbuildproduct_fast({[2 1; 1 2],[2 0;0 1]},[1 0 2]);
    val2=tbuildproduct_fast({[2 1; 1 2],[2 0;0 1]},[1 0 2],vv);
    if(norm(val1*vv-val2)>10e-12); vprintf('ERROR: tbuildproduct_fast() is broken.\n\n','cpr','err'); end; 
    
    %testtjsr %this file
    
    vprintf('tgallery\n');
    T=tgallery('rand_stochastic',5,2); if(~identifymatrix(T,0,'columnstochastic')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_doublestochastic',5,2); if(~identifymatrix(T,0,'doublestochastic')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_stochastic_neg',5,2); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_doublestochastic_neg',5,2); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_pm1',5,2); if(~identifymatrix(T,0,'pm1')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_bool',5,2); if(~identifymatrix(T,0,'bool')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_gauss',5,2); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_gauss',5,2,'sparse',.5); if(~identifymatrix(T,1,'sparsity')); vprintf('ERROR: tgallery() may be broken.\n\n','cpr',[.6 .4 .1]); end; 
    T=tgallery('rand_gauss',5,2,'int',1); if(~identifymatrix(T,1,'int')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand',5,2); if(~identifymatrix(T,0,'pos')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_equal',5,2); if(~identifymatrix(T,0,'pos')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_neg',5,2); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_equal_neg',5,2); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_unitary',5,2); if(~identifymatrix(T,1,'unitary')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_zero',5,2); if(~identifymatrix(T,1,'zero')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_colu_1',5,2); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_colu_0',5,2); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_corr_1',5,2); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_corr_0',5,2); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('rand_hess',5,2); if(~identifymatrix(T,0,'hessu')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('orthog_1',5,2); if(~identifymatrix(T,1,'unitary')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('orthog_2',5,2); if(~identifymatrix(T,1,'unitary')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('orthog_3',5,2); if(~identifymatrix(T,1,'unitary')); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('cex'); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('binary',5,2,10); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('binary2',5,2,10); if(~isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('mejstrik_119',5,2,10); if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    T=tgallery('mejstrik_Cn',5,2,10);if(isempty(T)); vprintf('ERROR: tgallery() is broken.\n\n','cpr','err'); end; 
    
    

    tgallery('rand_gauss',5,2,'seed',100,'sparse',.5,'bool');
    
    tjsrflag=true;
    figure;
    try
        vprintf('tjsr:..................\n     '); vprintf('|'); 
        tjsr(tgallery('rand_pm1',3,2,'seed',100),'ordering',{[1 2 2 2 2 2 2 2]'},'plot','info_rho','verbose',-1);
        if(~isequal(tjsr({[1],[2]},'verbose',verbose-1),2)); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;
        if(norm(tjsr({[1 -1; 0 1],[0 0; 1 0]},'verbose',verbose-1)-1.31950791077)>10e-10);  tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;
        %
        vprintf('|');
        AA={[1 1 0; 1 0 1; 1 0 0],[1 0 1; 1 1 0; 0 1 0]};
        [JSR,info]=tjsr(AA,'plot','polytope','proof',1,'verbose',-1);
        if(norm(JSR-1.83928675521)>1e-10); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;
        if(info.info.errorcode~=-10 || numel(JSR)>1); tjsrflag=false; vprintf('ERROR: tjsr() may be broken. Test tjsr() again yourself.\n\n','cpr','err'); end;
        %
        % stupid input tests
        vprintf('|');
        B={[1 1 0; -1 0 -1; 1 0 0],[1 0 -1; 1 -1 0; 0 1 0]};
        [JSR,info]=tjsr(B,'verbose',verbose-1); if(norm(JSR-1.39406936116)>1e-10); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        if(~isnan(tjsr({nan},'verbose',verbose-1))); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        if(~isinf(tjsr({inf},'verbose',verbose-1))); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        if(~isequal(tjsr({[1],[2]},'verbose',verbose-1),2)); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        if(~isequal(tjsr({[1],[-1]},'verbose',verbose-1),1)); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        if(~isequal(tjsr({[1 2; 1 2]},'verbose',verbose-1),3)); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        %
        AA={[1 1;0 1],[1 2;0 2]}; 
        if(~isequal(tjsr(AA,'verbose',verbose-1),2)); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        %
        AA={[1 1 1;0 1 -1;1 -1 -1], [0 0 1;-1 -1 0;1 1 -1]};
        vprintf('|'); JSR=tjsr(AA,'ordering',{[2]'},'verbose',-1,'maxiteration',10); if(norm(JSR-2)>10e-12 || numel(JSR)>1); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        vprintf('|'); JSR=tjsr(AA,'extravertex',{[.1 0 0]'},'verbose',-1,'invariantsubspace','none','maxiteration',10);  if(norm(JSR-2)>10e-12 || numel(JSR)>1); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        vprintf('|'); JSR=tjsr(AA,'ordering',{[1 2 2 2 2 2 2 2 2]', []'},'smpflag',[0 2],'v0', {[0.058585928823279 -0.687551547968790 0.723768303968635]', [.1 0 0]'},'verbose',-1,'invariantsubspace','none','maxiteration',10); if(norm(JSR-2)>10e-12 || numel(JSR)>1); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        %
        D7=daubechiesmatrix(7,'verbose',0);
        vprintf('|'); JSR=tjsr(D7,'verbose',-1,'maxiteration',10); if(norm(JSR-0.181695060514689)>10e-12 || numel(JSR)>1); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        vprintf('|'); JSR=tjsr(D7,'balancingvector',[1 1.02 .01 .01 .01 .01 .01],'verbose',verbose-1,'maxiteration',10,'invariantsubspace','none'); if(norm(JSR-0.181695060514703)>10e-12 || numel(JSR)>1); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        vprintf('|'); JSR=tjsr(D7,'nobalancing','verbose',verbose-1,'maxiteration',100,'epspolytope',-1e-8); if(numel(JSR)==1); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        vprintf('|'); JSR=tjsr(D7,'nobalancing','delta',.99999,'verbose',verbose-1,'maxiteration',10); if(norm(JSR-0.181695060514703)>10e-4 || numel(JSR)==1); tjsrflag=false; vprintf('ERROR: tjsr() is broken.\n','cpr','err'); end;    
        vprintf('|'); JSR=tjsr(D7,'alwaysout','maxiteration',3,'verbose',verbose-1,'fastnorm',0);
        %
        T=tgallery('rand_gauss',20,2,'rho','seed',100,'verbose',verbose-1);
        vprintf('|'); tjsr(T,'naturalselectiontype',2,'plot','norm','verbose',-1,'maxiteration',4);
        vprintf('|'); figure; tjsr(T,'naturalselectiontype',3,'plot','L','verbose',-1,'maxiteration',5);
        vprintf('|'); tjsr(T,'plot','info_normest_norm_rho','verbose',-1,'maxiteration',6);
        vprintf('|'); tjsr(T,'plot','polytope','maxiteration',5,'verbose',-1);
        %
        T=tgallery('mejstrik_119',20,2);
        vprintf('|'); tjsr(T,'plot','tree','verbose',-1,'maxiteration',5);
        vprintf('|'); tjsr(T,'plot','polytope','maxiteration',5,'verbose',-1);
        vprintf('|'); tjsr(T,'plot','o','maxiteration',5,'verbose',-1);
    catch
        tjsrflag=false;
        vprintf('ERROR: tjsr() is broken.\n','cpr','err'); 
    end
    vprintf('\n');
    
    %tjsr_ %these files are not tested
    
    vprintf('tliftproduct\n');
    val=unique(cell2mat(tliftproduct({2 3 5}, 2)));
    if(~isequal(val, [1 2 3 4 5 6 9 10 15 25])); vprintf('ERROR: tliftproduct() us broken.\n\n','cpr','err'); end;
    
    vprintf('tliftsemidefinite\n');
    val=tliftsemidefinite({[1 1;0 1] [1 0; 1 0]}, 2);
    if(~isequal(val, {[1 4 4 2 4 1;0 1 2 1 3 1;0 0 1 0 2 1;0 0 0 1 2 1;0 0 0 0 1 1;0 0 0 0 0 1]; [ones(6,1) zeros(6,5)]})); vprintf('ERROR: tliftsemidefinite() broken.\n\n','cpr','err'); end;
    
    vprintf('trho\n');
    MM=tgallery('rand_gauss',10,2,'rho');
    if(norm(trho(MM{1})-1)>10e-12); vprintf('ERROR: trho() may be broken.\n\n','cpr','err'); end;
    trho(sparse(randi(2,2)));
    trho({[] [1] [1 1;1 1] sparse([1 0;0 0])});
    trho([]);
    trho({1 [1 1;0 1]});
    
    %Gurobi functions
    vprintf('Gurobi\n');
    try      
        evalc('gurobi_setup');
        model.A = sparse([1 1 0; 0 1 1]);
        model.obj = [1 2 3];
        model.modelsense = 'Max';
        model.rhs = [1 1];
        model.sense = [ '<' '<'];
        params.outputflag=0;

        gurobiresult = gurobi(model,params);
        if(~isequal(gurobiresult.x,[ 1 0 1].')); 
            vprintf('ERROR: Gurobi seems to be installed, but does not work.\n','cpr','err');
            vprintf('ERROR: The package will work nevertheless, but magnitudes slower. It is strongly recommended to fix this issue.\n\n','cpr','err');
        end
       % vprintf('There should be a lot of text output.\n\n','cpr',[.1 .4 .1]);
    catch
        vprintf('ERROR: Gurobi is broken / not installed.\n','cpr','err');
        vprintf('ERROR: The package will work nevertheless, but magnitudes slower. It is strongly recommended to install Gurobi.\n\n','cpr','err');
    end
    
    %Sedumi functions
%     vprintf('sedumi\n');
%     try
%         load('arch0','At','b','c','K');
%         pars.fid=0;
%         [x,y,info] = sedumi(At,b,c,K,pars);
%         %vprintf('There should be a lot of text output.\n\n','cpr',[.1 .4 .1]);
%     catch        
%         vprintf('ERROR: SeDuMi is broken / not installed.\n','cpr','err');
%         vprintf('ERROR: This does not mean, that the packages do not work, since the packages use only a subset of functions of SeDuMi.\n\n','cpr','err');
%     end
    
%     %JSR-toolbox functions
%     vprintf('JSR-toolbox\n');
%     try
%         AA{1}=[1 2; -2 1];AA{2}=[1 2; -1 1];
%         opts = jsrsettings('logfile',0,'verbose',verbose);
%         valjsr=jsr(AA,opts);
%         valtjsr=tjsr(AA,'verbose',verbose);
%         if(norm(min(valjsr-valtjsr))>0.1); 
%             vprintf('ERROR: JSR-toolbox may be broken.\n','cpr','err');
%         end
%         %vprintf('There should be text output.\n\n','cpr',[.1 .4 .1]);
%     catch
%         vprintf('ERROR: The JSR toolbox is broken / not installed.\n','cpr','err');
%         vprintf('ERROR: This does not mean, that the packages do not work, since the packages use only a subset of functions of The JSR toolbox.\n\n','cpr','err');
%     end
    
    if(~tjsrflag)
        vprintf('BAD NEWS: The most important function tjsr works not as it should.\n','cpr','err');
    end
end


% clc
% profile off;
% profile clear;
% profile on;
% setuptjsr(1);
% profile off;
% profile viewer;

function dummy; end %#ok<DEFNU> %Generates an error, if the 'end' of a function is missing. 

