function setuptmisc(flag)
% setuptmisc( [flag] )
% Tests the package tmisc
% if flag is 1, then the functions in the package are tested.
%
% Written by: tommsch, 2018

%#ok<*NOPRT>
%#ok<*ASGLU>
%#ok<*NASGU>

if(nargin>=1 && flag); selftesttmisc; return; end;

vprintf('Either start the function with argument <1> or read the source code of this file.\n'); 
return; 
2; %#ok<UNRCH>

end

function selftesttmisc
    vprintf('tmisc package will be tested. All output on the screen should be followed by an explanation.\n','cpr',[.1 .4 .1]);
    vprintf('If there is some error or un-commented output, the test fails.\n\n','cpr',[.1 .4 .1]);
    
    vprintf('cprintf\n');
    cprintf('_green','This should be underlined green text.\n');
    
    vprintf('flatten\n');
    val=flatten({{2 3 {4; 5} 6} {{{7}}}});
    if(~isequal(val, {2 3 4 5 6 7})); vprintf('ERROR: flatten() is broken.\n\n','cpr','err'); end;    
    flatten({2});
    if(~iscell(flatten([2 3]))); vprintf('ERROR: flatten() is broken.\n\n','cpr','err'); end;    
    
    vprintf('grCenter\n');
    G=[1 2;2 5; 5 6; 2 3; 3 4; 4 7; 4 8; 8 9];
    [cc,wei]=grCenter(G);
    if(~isequal(cc, 3)); vprintf('ERROR: grCenter() is broken.\n\n','cpr','err'); end;    
    [cc,wei]=grCenter(G,'edge');
    G=[1 2; 2 3; 4 5; 2 4];    
    cc=grCenter(G);    
    if(~isequal(sort(cc),[2 4])); vprintf('ERROR: grCenter() is broken.\n\n','cpr','err'); end;    
    
    vprintf('grVerCover\n');
    G=[1 2; 1 3; 1 4; 2 5; 1 6];
    G=digraph(G(:,1),G(:,2)); %plot(G)
    val=grVerCover(G);
    if(numel(val)~=3 || ~searchincellarray([1 2],val,1) || ~searchincellarray([1 5],val,1) || ~searchincellarray([2 3 4 6],val,1)); vprintf('ERROR: grVerCover() is broken.\n\n','cpr','err'); end;    
    G=[1 1; 1 3; 1 2; 2 1];
    G=digraph(G(:,1),G(:,2));  %plot(G)
    val=grVerCover(G);
    if(~isequal(val,{1})); vprintf('ERROR: grVerCover() is broken.\n\n','cpr','err'); end;    

    
    vprintf('identifymatrix\n');
    val=identifymatrix({[1 2 3; 0 2 1; 0 0 1],[2]});
    if(~isequal(val.nan,0)); vprintf('ERROR: identifymatrix() is broken.\n\n','cpr','err'); end;    
    val=identifymatrix(1);
    
    vprintf('intersectspace\n');
    [in,dim]=intersectspace([1 -1 0 0; 1 0 -2 1; 0 1 -2 -1]',[1 1 0 0; 1 0 -1 0]');
    if(~isequal(rank(in),1) || ~isequal(dim,1));  vprintf('ERROR: intersectspace() is broken.\n\n','cpr','err'); end;    
    [in,dim]=intersectspace([1 0 0; 0 1 0;1 1 0]');
    if(~isequal(dim,2));  vprintf('ERROR: intersectspace() is broken.\n\n','cpr','err'); end;    
    [in,dim]=intersectspace([1 0 0]',[0 1 0;1 -1 0]',[0 1 1;0 1 -1]',[1 1 1;1 1 -1; 1 -1 1]');
    if(~isequal(dim,0));  vprintf('ERROR: intersectspace() is broken.\n\n','cpr','err'); end;        
    [in,dim]=intersectspace(sym([1 0 0]'),[1 1 0; 1 -1 0]');
    if(~isequal(dim,1));  vprintf('ERROR: intersectspace() is broken.\n\n','cpr','err'); end;            
    
    vprintf('issquare\n');
    if(~isequal(issquare(rand(3,3,3)),true)); vprintf('ERROR: issquare() is broken.\n\n','cpr','err'); end;
    if(~isequal(issquare(rand(3,4)),false)); vprintf('ERROR: issquare() is broken.\n\n','cpr','err'); end;
    
    vprintf('issym\n');
    if(~isequal(issym([sym('23.2') 0]),true)); vprintf('ERROR: issym() is broken.\n\n','cpr','err'); end;
    
    vprintf('iswholenumber\n');
    if(~isequal(iswholenumber([2 2.1 inf 0]),[true false false true])); vprintf('ERROR: iswholenumber() is broken.\n\n','cpr','err'); end;
    if(~isequal(iswholenumber(@sum),false)); vprintf('ERROR: iswholenumber() is broken.\n\n','cpr','err'); end;
    
    vprintf('lexicographic\n');
    val=lexicographic( [0 1 0; 0 1 2; 3 1 1] ,2);
    if(~isequal(val,[1 0 0;1 2 0;1 1 3])); vprintf('ERROR: lexicographic() is broken.\n\n','cpr','err'); end;
    val=lexicographic( [0 1 0; 0 1 2; 3 1 1]);
    val=lexicographic( [0 1 0; 0 1 2; 3 1 1],'inf');

    vprintf('liminf\n');
    if(~isequal(liminf([10 1 9 2 8 3 7 4 6]),[1 1 2 2 3 3 4 4 6])); vprintf('ERROR: liminf() is broken.\n\n','cpr','err'); end;        
    
    vprintf('limsup\n');
    if(~isequal(limsup([10 1 9 2 8 3 7 4 6]),[10 9 9 8 8 7 7 6 6])); vprintf('ERROR: limsup() is broken.\n\n','cpr','err'); end;
    
    vprintf('mergeinterval\n');
    [lower, upper] = mergeinterval([0 1 2 3 4],[1.5 1.6 3.5 3 5]);
    v = mergeinterval(sym([0 1.5 1 1.6 2 3.5 3 3 4 5]));
    C = mergeinterval({[0 1.5];[1 1.6];[2 3.5];[3 3];[4 5]});
    M = mergeinterval([0 1.5; 1 1.6; 2 3.5; 3 3; 4 5]);
    C = mergeinterval([0 1.5; 1 1.6; 2 3.5; 3 3; 4 5],'output','C');
    if(~isequal(lower,[0 2 4]) || ~isequal(upper,[1.6 3.5 5])); vprintf('ERROR: mergeinterval() is broken.\n\n','cpr','err'); end;    
    [val1,val2]=mergeinterval(); mergeinterval({[]}); mergeinterval([]); 
    if(~isequal(mergeinterval([0 2; 1 -1],'output','M'),[0 2])); vprintf('ERROR: mergeinterval() is broken.\n\n','cpr','err'); end;    
    if(~isequal(mergeinterval([0 2; -1 1],'output','M','switch'),[-1 2])); vprintf('ERROR: mergeinterval() is broken.\n\n','cpr','err'); end;    
    if(~isequal(mergeinterval([-inf inf],'output','M','switch'),[-inf inf])); vprintf('ERROR: mergeinterval() is broken.\n\n','cpr','err'); end;        
    if(~isequal(mergeinterval([],'output','M','switch'),zeros(0,2))); vprintf('ERROR: mergeinterval() is broken.\n\n','cpr','err'); end;        
    
    vprintf('mixvector\n');
    val=sortrows(mixvector(0:1, 2)')';
    if(~isequal(val,sortrows([0 0 1 1;0 1 0 1]')')); vprintf('ERROR: mixvector() is broken.\n\n','cpr','err'); end;
    if(~isequal(mixvector([-1 1], 3,0),8)); vprintf('ERROR: mixvector() is broken.\n\n','cpr','err'); end;
    if(~isequal(mixvector([-1 1], 3,1),[-1;-1;-1])); vprintf('ERROR: mixvector() is broken.\n\n','cpr','err'); end;
    [~,val]=mixvector(0:1, 2);
    if(~isequal(val,4)); vprintf('ERROR: mixvector() is broken.\n\n','cpr','err'); end;

    vprintf('nestedcellfun\n');
    if(~isequal(nestedcellfun(@(x) sum(x),{[1 2],[2 3 4]}),[3 9])); vprintf('ERROR: nestedcellfun() is broken.\n\n','cpr','err'); end;
    val=nestedcellfun(@(x) ndimsm(x),{{[1 2 1]},[1 ;2 ]},'UniformOutput',false);
    if(~isequal(val,{{2},1})); vprintf('ERROR: nestedcellfun() is broken.\n\n','cpr','err'); end;
    
    vprintf('nondiag\n');
    val=nondiag({[1 2] [2 3] ;[1] [40] },1);
    if(~isequal(val,{[1 2] []; 1 40})); vprintf('ERROR: nondiag() is broken.\n\n','cpr','err'); end;
    nondiag([1 2 3; 3 2 3; 3 4 5]);
    try; %#ok<TRYNC>
        nondiag([1 2 3; 3 2 3; 3 4 5],-inf);
        nondiag([1 2 3; 3 2 3; 3 4 5],inf);
    end;
    
    vprintf('normalizematrix\n');
    MM=[1 2 3; 0 -1 2];
    val=normalizematrix(MM,'colsum');
    if(norm(sum(val,1)-1)>10e-12); vprintf('ERROR: normalizematrix() is broken.\n\n','cpr','err'); end;
    val=normalizematrix(MM,'rowsum');
    if(norm(sum(val,2)-1)>10e-12); vprintf('ERROR: normalizematrix() is broken.\n\n','cpr','err'); end;
    normalizematrix(MM,'dirsum',3);
    val=normalizematrix(MM,'dirmax',1);
    if(norm(val-[1 1 1;0 -.5 2/3])>10e-12); vprintf('ERROR: normalizematrix() is broken.\n\n','cpr','err'); end;    
    val=normalizematrix(MM,'dirnorm',[2 2]);
    if(norm(sum(val.^2,2)-[1;1])>10e-12); vprintf('ERROR: normalizematrix() is broken.\n\n','cpr','err'); end;
    MM=[-1 2; 0 3];
    val=normalizematrix(MM,'rho');
    if(norm(rho(val)-1)>10e-12); vprintf('ERROR: normalizematrix() is broken.\n\n','cpr','err'); end;
    val=normalizematrix(MM,'norm',inf);
    if(norm(norm(val,inf)-1)>10e-12); vprintf('ERROR: normalizematrix() is broken.\n\n','cpr','err'); end;    
    if(~isequal(normalizematrix(MM,'binary'),[1 1;0 1])); vprintf('ERROR: normalizematrix() is broken.\n\n','cpr','err'); end;    
    %XX 'dotprod' is missing
    if(~isequal(normalizematrix(MM,'positive',2),[1 -2;0 3])); vprintf('ERROR: normalizematrix() is broken.\n\n','cpr','err'); end;    
    
    vprintf('num2color\n');
    vprintf('This must be colored text.\n','cpr',num2color(1000,100));
    vprintf('This must be colored text too.\n','cpr',num2color(1000));
        
    vprintf('removezero\n');
    AA=[0 0 0 0;0 1 0 0;0 0 0 0;0 2 0 3];
    if(~isequal(removezero(AA,'compact',1),[0 1 0 3;0 2 0 0])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,'compact',2),[0 0;1 0;0 0;2 3])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,'compact',[1 2]),[1 3; 2 0])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,[1 2]),[1 0; 2  3])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,[]),AA)); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,'border'),[1 0 0;0 0 0;2 0 3])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,'outside'),[0 0 0 0; 0 1 0 0;0 0 0 0;0 2 0 3])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,'inside'),[1 0 0;0 0 0;2 0 3])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,'all'),[1;2;3])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,'left'),[0 0 0; 1 0 0;0 0 0;2 0 3])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,'right'),AA)); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,'top'),[0 1 0 0;0 0 0 0;0 2 0 3])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    if(~isequal(removezero(AA,'bottom'),AA)); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    AA=[0 1 0; 0 2 0];
    if(~isequal(removezero(AA,[2],'keepdim'),[1 0;2 0])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    [~,idx]=removezero(AA,[2],'keepdim');
    if(~isequal(idx,[1;2])); vprintf('ERROR: removezero() is broken.\n\n','cpr','err'); end;
    
    vprintf('repcell\n');
    if(~isequal(repcell(10 ,2),{10 10;10 10})); vprintf('ERROR: repcell() is broken.\n\n','cpr','err'); end;
    
    vprintf('savetocellarray\n');
    val=savetocellarray([1 2 3],logical([1 1 0 0 1 ]),{[-1 -2], [-3 -4 -5]});
    if(~isequal(val,{[1 2], [-3 -4 3]})); vprintf('ERROR: savetocellarray() is broken.\n\n','cpr','err'); end;
    
    vprintf('searchincellarray\n');
    [found, ioo]=searchincellarray([1 2 3 4], {[1 2; 3 4],[1 2 3 4],[2 3 1]',[5]}, 0);
    if(~isequal(found,true) || ~isequal(ioo,2)); vprintf('ERROR: searchincellarray() is broken.\n\n','cpr','err'); end;
    
    vprintf('setplus\n');
    setplus([1 0; 1 1],[10 10; 20 20; 30 30],'rows');
    setplus(1);
    val=setplus([1 0; 1 1]',[10 10; 20 20; 30 30]');
    if(~isequal(val,[11 11 21 21 31 31;10 11 20 21 30 31])); vprintf('ERROR: setplus() is broken.\n\n','cpr','err'); end;    
    val=setplus([1 0],[0 1],'stable');
    if(~isequal(val,[1 0 2])); vprintf('ERROR: setplus() is broken.\n\n','cpr','err'); end;    
    val=setplus([1 0],[0 1],[0],[1],'nounique');
    if(~isequal(sort(val),[1 2 2 3])); vprintf('ERROR: setplus() is broken.\n\n','cpr','err'); end;  
    
    vprintf('subsco\n');
    val=subsco([1 2 3; 4 5 6],[1 2; 2 3]');
    if(~isequal(val,[2 6])); vprintf('ERROR: subsco() is broken.\n\n','cpr','err'); end;
    val=subsco([1 2 3; 4 5 6],[1 2; 2 3]',[-1 -1]);
    if(~isequal(val,[1 -1 3;4 5 -1])); vprintf('ERROR: subsco() is broken.\n\n','cpr','err'); end;
    try; val=subsco([1 2 3; 4 5 6],[1 2; 4 3]');
         vprintf('ERROR: subsco() is broken.\n\n','cpr','err');
    catch;
        val=subsco([1 2 3; 4 5 6],[1 2; 4 3]','save');
    end
    try;
        val=subsco([1 2 3; 4 5 6],[1 2; 1 2]',[1 2],'save');
        vprintf('ERROR: subsco() is broken.\n\n','cpr','err');
    catch;
    end
    
    vprintf('tbuildproduct\n'); %XX Hier weitermachen
    MAT={[0.8 0.2; 0.3 0.7],[0.4 0.6;0.5 0.5]};
    SOL=1/21*[11 10;11 10];
    val=tbuildproduct(MAT, {[],[1 2]});
    if(norm(val-SOL)>10e-12); vprintf('ERROR: tbuildproduct() is broken.\n\n','cpr','err'); end;
    val=tbuildproduct(MAT, {[],[1 2]},'sym');
    if(norm(val-SOL)>10e-12); vprintf('ERROR: tbuildproduct() is broken.\n\n','cpr','err'); end;
    val=tbuildproduct(MAT, {[],[2 1]},'reverse');
    if(norm(val-SOL)>10e-12); vprintf('ERROR: tbuildproduct() is broken.\n\n','cpr','err'); end;
    tbuildproduct(MAT, {[],[1 2]},'l',1);
    SOL=1/20*[10 10;11 9];
    val=tbuildproduct(MAT, [1 2]);
    if(norm(val-SOL)>10e-12); vprintf('ERROR: tbuildproduct() is broken.\n\n','cpr','err'); end;
    val=tbuildproduct(MAT, {[1 2],[]});
    if(norm(val-SOL)>10e-12); vprintf('ERROR: tbuildproduct() is broken.\n\n','cpr','err'); end;    
        
    vprintf('tif\n');
    val=[tif(true, 1, 2) tif(false, 1, 2)];
    if(~isequal(val,[1 2])); vprintf('ERROR: tif() is broken.\n\n','cpr','err'); end;
    
    vprintf('unflatten/flatten\n');
    C={[2 3 4] [5 ; 5 ; 5 ] {3 7; 4 6}; 40 50 {60  80}};
    if(~isequal(C,unflatten(flatten(C),C))); vprintf('ERROR: unflatten() is broken.\n\n','cpr','err'); end;
    C={1}; if(~isequal(C,unflatten(flatten(C),C))); vprintf('ERROR: unflatten() is broken.\n\n','cpr','err'); end;    
    C=[1 2 3;4 5 6]; if(~isequal(C,unflatten(flatten(C),C))); vprintf('ERROR: unflatten() is broken.\n\n','cpr','err'); end;        
    C={1 {} }; if(~isequal(C,unflatten(flatten(C),C))); vprintf('ERROR: unflatten() is broken.\n\n','cpr','err'); end;    
    C={}; if(~isequal(C,unflatten(flatten(C),C))); vprintf('ERROR: unflatten() is broken.\n\n','cpr','err'); end;    
    C={{}}; if(~isequal(C,unflatten(flatten(C),C))); vprintf('ERROR: unflatten() is broken.\n\n','cpr','err'); end;    
    C=[]; if(~isequal(C,unflatten(flatten(C),C))); vprintf('ERROR: unflatten() is broken.\n\n','cpr','err'); end;    
    C={1 2 3}; val=unflatten(10,C,'assign'); if(~isequal(val,{10 10 10})); vprintf('ERROR: unflatten() is broken.\n\n','cpr','err'); end;
    C={}; val=unflatten(10,C,'assign'); if(~isequal(val,{})); vprintf('ERROR: unflatten() is broken.\n\n','cpr','err'); end;
    C={{}}; val=unflatten(10,C,'assign'); if(~isequal(val,{{}})); vprintf('ERROR: unflatten() is broken.\n\n','cpr','err'); end;
    
    vprintf('uniquecell\n');
    val=uniquecell({1 2 1 [1 2] [1 3] [1 2]})';
    if(numel(val)~=4); vprintf('ERROR: uniquecell() is broken.\n\n','cpr','err'); end;
    [Auc, idxc ,idx2c]=uniquecell({1 2 3 1 3 4 5});
    [Aua, idxa ,idx2a]=unique(    [1 2 3 1 3 4 5]);
    if(~isequal(cell2mat(Auc),Aua) || ~isequal(idxc,idxa) || ~isequal(idx2c,idx2a)); vprintf('ERROR: uniquecell() is broken.\n\n','cpr','err'); end;
    
    vprintf('vdisp\n');
    val=vdisp({[1 2 3]});
    if(~ischar(val) || ~strfind(val,'1') || ~strfind(val,'2') || ~strfind(val,'3'));  vprintf('ERROR: vdisp() is broken.\n\n','cpr','err'); end; %#ok<STRIFCND>
    ST=struct; ST.There='should'; ST.be='a'; ST.struct=@displayed; vdisp(ST);
    val=vdisp({[1 2 3]},'OLD');
    
    vprintf('vprintf\n');
    vprintf('Below the blue text must be green text.\n','cpr','blue')
    vprintf('ERROR: vprintf() is broken.\n\n','imp',[3 2],'cpr','err');
    vprintf('Above the green text must be blue text.\nBelow should be a placeholder.\n','cpr',[0 .6 0],'imp',[0 0]);
    vprintf('ERROR: vprintf() is broken.\n\n','cpr','err','sze',[10 100]);
    vprintf('Below this line should be a matrix.\n%v\n',[1 2 3;4 5 6],'cpr',[0 .6 0]);
    vprintf('One, Two Three: %i %f %v\n',1,2,{{3}});
    vprintf('And again the struct: %v\n',ST,'cpr',[0 .6 0]);
    val=vprintf('Some text again.','str','OLD');vprintf('\n');
    if(~isequal(val,'OLDSome text again.')); vprintf('ERROR: vprintf() is broken.\n\n','cpr','err'); end;
    vprintf('100 Percent: %i%% and very compact printing of matrix:\n%r\n\n',100,[100:120],'cpr',[0 .6 0]);
    try; vprintf('ERROR: vprintf() is broken.%i\n\n','cpr','err');
    catch
    end
    
end

function dummy; end %#ok<DEFNU> %Generates an error, if the 'end' of a function is missing. 
